package de.hft.stuttgart.citydoctor2.webservice.endpoints;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;

import javax.ws.rs.core.HttpHeaders;

import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.mime.HttpMultipartMode;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.HttpClients;
import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;
import org.junit.Ignore;
import org.junit.Test;

public class ModelsEndpointTest {

	@Test
	public void testRetrievingModels() throws ClientProtocolException, IOException {
		HttpClient client = HttpClients.createMinimal();
		LoginValues login = TestUtils.login(client);

		HttpGet get = new HttpGet("http://localhost:8080/CityDoctorWebService/rest/models");
		get.addHeader(HttpHeaders.AUTHORIZATION, "Bearer " + login.getToken());

		HttpResponse response = client.execute(get);

		assertEquals(200, response.getStatusLine().getStatusCode());
		JSONTokener tokener = TestUtils.tokenizeReponse(response);
		JSONArray models = new JSONArray(tokener);

		for (Object o : models) {
			assertTrue(o instanceof JSONObject);
		}
		assertNotNull(models);

	}

	@Test
	public void testUploadModel() throws ClientProtocolException, IOException {
		HttpClient client = HttpClients.createMinimal();
		LoginValues login = TestUtils.login(client);

		HttpPost post = new HttpPost("http://localhost:8080/CityDoctorWebService/rest/models");
		post.addHeader(HttpHeaders.AUTHORIZATION, "Bearer " + login.getToken());
		File file = new File("C:\\Job\\CityGML Files\\1-stoeckach_multisurface_only.xml");
//		File configFile = new File("src\\test\\resources\\testConfig.yml");
		MultipartEntityBuilder entityBuilder = MultipartEntityBuilder.create();
		entityBuilder.setMode(HttpMultipartMode.BROWSER_COMPATIBLE);
		entityBuilder.addBinaryBody("file", file);
//		entityBuilder.addBinaryBody("configFile", configFile);
		post.setEntity(entityBuilder.build());

		HttpResponse response = client.execute(post);
		int httpStatus = response.getStatusLine().getStatusCode();
		assertEquals(200, httpStatus);
		JSONTokener tokener = TestUtils.tokenizeReponse(response);
		JSONObject model = new JSONObject(tokener);
		assertNotNull(model);
		assertEquals("1-stoeckach_multisurface_only.xml", model.getString("name"));
	}

	@Ignore
	@Test
	public void downloadPdfFile() throws ClientProtocolException, IOException {
		HttpClient client = HttpClients.createMinimal();
		LoginValues login = TestUtils.login(client);

		HttpGet get = new HttpGet(
				"http://localhost:8080/CityDoctorWebService/rest/" + login.getId() + "/models/15/pdf");
		get.addHeader(HttpHeaders.AUTHORIZATION, "Bearer " + login.getToken());

		HttpResponse response = client.execute(get);

		assertEquals(200, response.getStatusLine().getStatusCode());

		Path path = Files.createTempFile(null, null);
		Files.copy(response.getEntity().getContent(), path, StandardCopyOption.REPLACE_EXISTING);
		assertTrue(path.toFile().exists());
	}

	@Ignore
	@Test
	public void downloadXmlFile() throws ClientProtocolException, IOException {
		HttpClient client = HttpClients.createMinimal();
		LoginValues login = TestUtils.login(client);

		HttpGet get = new HttpGet(
				"http://localhost:8080/CityDoctorWebService/rest/" + login.getId() + "/models/15/xml");
		get.addHeader(HttpHeaders.AUTHORIZATION, "Bearer " + login.getToken());

		HttpResponse response = client.execute(get);

		assertEquals(200, response.getStatusLine().getStatusCode());

		Path path = Files.createTempFile(null, null);
		Files.copy(response.getEntity().getContent(), path, StandardCopyOption.REPLACE_EXISTING);
		assertTrue(path.toFile().exists());
	}
	
	@Test
	public void testGetBuildings() throws ClientProtocolException, IOException {
		HttpClient client = HttpClients.createMinimal();
		LoginValues login = TestUtils.login(client);

		HttpGet get = new HttpGet("http://localhost:8080/CityDoctorWebService/rest/models/5/buildings?from=0");
		get.addHeader(HttpHeaders.AUTHORIZATION, "Bearer " + login.getToken());
		
		HttpResponse response = client.execute(get);
		
		String readCompleteReponse = TestUtils.readCompleteReponse(response);
		System.out.println(readCompleteReponse);

//		JSONTokener tokener = TestUtils.tokenizeReponse(response);
//		JSONArray models = new JSONArray(tokener);

//		for (Object o : models) {
//			System.out.println(o);
//		}

	}

}
